package electionSimulator;

/**
 * This class represents a state in the election. In stores the current poll
 * numbers for two candidates plus the number of electoral votes for the state.
 * It can also (optionally) store the state name.
 * 
 * @author Curt Clifton. Created Sep 23, 2008.
 */
public class State {
	private String stateName;
	private int electoralVotes;
	private double candidateAOdds;
	private double candidateBOdds;

	/**
	 * Constructs a state for the election.
	 * 
	 * @param stateName Name of the state
	 * @param electoralVotes
	 *            the electoral votes at stake in the state
	 * @param candidateAOdds
	 *            the odds of candidate A winning the state, expressed as a
	 *            probability between 0.0 and 1.0
	 * @param candidateBOdds
	 *            the odds of candidate B winning the state, expressed as a
	 *            probability between 0.0 and 1.0
	 */
	public State(String stateName, int electoralVotes, double candidateAOdds,
			double candidateBOdds) {
		this.stateName = stateName;
		this.electoralVotes = electoralVotes;
		this.candidateAOdds = candidateAOdds;
		this.candidateBOdds = candidateBOdds;
	}

	/**
	 * @return the number of electoral votes at stake for this state
	 */
	public int getElectoralVotes() {
		return this.electoralVotes;
	}

	/**
	 * @return the state's name
	 */
	public String getName() {
		return this.stateName;
	}

	/**
	 * Simulates a single election in this state and returns the result. A
	 * positive number indicates the number of electoral votes earned by
	 * candidate A. A negative number indicates the number of electoral votes
	 * earned by candidate B. Assumes a third party candidate cannot win and 
	 * allocates the undecided voters evenly.
	 * 
	 * @return electoral votes
	 */
	public int simulate() {
		/*
		 * Generates a random number between 0.0 and 1.0. Uses candidate odds to
		 * decide the outcome of the election.
		 */
		double undecided = 1.0 - (this.candidateAOdds + this.candidateBOdds);
		double rand = Math.random();
		if (rand < this.candidateAOdds + (undecided / 2.0)) {
			// A wins
			return this.electoralVotes;
		} else {
			// B wins
			return -this.electoralVotes;
		}
	}
}
